@@ -399,6 +399,8 @@ destroy_clock(GtkWidget * widget, ClockData *cd)
 		gconf_client_notify_remove (
 				client, cd->listeners [i]);
 
+	g_object_unref (G_OBJECT (client));
+
 	if (cd->timeout > 0) {
 		g_source_remove (cd->timeout);
 		cd->timeout = 0;
@@ -430,6 +432,15 @@ close_on_escape (GtkWidget   *widget,
 	return FALSE;
 }
 
+static gboolean
+delete_event (GtkWidget   *widget,
+	      GdkEvent    *event,
+	      ClockData   *cd)
+{
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cd->toggle), FALSE);
+	return TRUE;
+}
+
 static GtkWidget *
 create_calendar (ClockData *cd,
 		 GdkScreen *screen)
@@ -439,16 +450,14 @@ create_calendar (ClockData *cd,
 
 	window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
 
-	/* set dialog type so it will skip the tasklist,
-	 * kinda broken. This window should really be override
-	 * redirect I suppose. But I hate override redirect
-	 * windows that don't have a pointer/keyboard grab.
-	 */
-	gtk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_DOCK);
 	gtk_window_set_decorated (window, FALSE);
 	gtk_window_set_resizable (window, FALSE);
 	gtk_window_stick (window);
 	gtk_window_set_title (window, _("Calendar"));
+
+	g_signal_connect (window, "delete_event",
+			  G_CALLBACK (delete_event), cd);
 			
 	g_signal_connect (window, "key_press_event",
 			  G_CALLBACK (close_on_escape), cd);
@@ -469,10 +478,12 @@ present_calendar_popup (ClockData *cd,
 {
 	GtkRequisition  req;
 	GdkScreen      *screen;
+	GdkRectangle    monitor;
 	int             button_w, button_h;
-	int             screen_w, screen_h;
 	int             x, y;
 	int             w, h;
+	int             i, n;
+	gboolean        found_monitor = FALSE;
 		
 	/* Get root origin of the toggle button, and position above that. */
 	gdk_window_get_origin (button->window, &x, &y);
@@ -487,11 +498,25 @@ present_calendar_popup (ClockData *cd,
 
 	screen = gtk_window_get_screen (GTK_WINDOW (window));
 
-	/* FIXME use xinerama extents for xinerama containing
-	 * the applet.
-	 */
-	screen_w = gdk_screen_get_width (screen);
-	screen_h = gdk_screen_get_height (screen);
+	n = gdk_screen_get_n_monitors (screen);
+	for (i = 0; i < n; i++) {
+		gdk_screen_get_monitor_geometry (screen, i, &monitor);
+		if (x >= monitor.x && x <= monitor.x + monitor.width &&
+		    y >= monitor.y && y <= monitor.y + monitor.height) {
+			found_monitor = TRUE;
+			break;
+		}
+	}
+
+
+	if ( ! found_monitor) {
+		/* eek, we should be on one of those xinerama
+		   monitors */
+		monitor.x = 0;
+		monitor.y = 0;
+		monitor.width = gdk_screen_get_width (screen);
+		monitor.height = gdk_screen_get_height (screen);
+	}
 		
 	/* Based on panel orientation, position the popup.
 	 * Ignore window gravity since the window is undecorated.
@@ -501,23 +526,23 @@ present_calendar_popup (ClockData *cd,
 	switch (cd->orient) {
 	case PANEL_APPLET_ORIENT_RIGHT:
 		x += button_w;
-		if ((y + h) > screen_h)
-			y -= (y + h) - screen_h;
-			break;
+		if ((y + h) > monitor.y + monitor.height)
+			y -= (y + h) - (monitor.y + monitor.height);
+		break;
 	case PANEL_APPLET_ORIENT_LEFT:
 		x -= w;
-		if ((y + h) > screen_h)
-			y -= (y + h) - screen_h;
+		if ((y + h) > monitor.y + monitor.height)
+			y -= (y + h) - (monitor.y + monitor.height);
 		break;
 	case PANEL_APPLET_ORIENT_DOWN:
 		y += button_h;
-		if ((x + w) > screen_w)
-			x -= (x + w) - screen_w;
+		if ((x + w) > monitor.x + monitor.width)
+			x -= (x + w) - (monitor.x + monitor.width);
 		break;
 	case PANEL_APPLET_ORIENT_UP:
 		y -= h;
-		if ((x + w) > screen_w)
-			x -= (x + w) - screen_w;
+		if ((x + w) > monitor.x + monitor.width)
+			x -= (x + w) - (monitor.x + monitor.width);
 		break;
 	}
 		
@@ -805,13 +830,16 @@ config_date (BonoboUIComponent *uic,
 
 	screen = gtk_widget_get_screen (cd->applet);
 
-	/* FIXME add GST, etc. */
+	/* FIXME add other time config tools. */
 	if (cd->config_tool && try_config_tool (screen, cd->config_tool))
 		return;
 
 	else if (try_config_tool (screen, "redhat-config-date"))
 		return;
 		
+	else if (try_config_tool (screen, "time-admin"))
+		return;
+		
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
@@ -1034,6 +1062,8 @@ setup_gconf (ClockData *clock)
 				(GConfClientNotifyFunc) config_tool_changed,
 				clock, NULL, NULL);
 	g_free (key);
+
+	g_object_unref (G_OBJECT (client));
 }
 
 static gboolean
@@ -1144,6 +1174,8 @@ setup_writability_sensitivity (ClockData *clock, GtkWidget *w, GtkWidget *label,
 	}
 
 	g_free (fullkey);
+
+	g_object_unref (G_OBJECT (client));
 }
 
 static void
@@ -1316,6 +1348,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	gtk_window_set_screen (GTK_WINDOW (cd->props),
 			       gtk_widget_get_screen (cd->applet));
 	gtk_container_set_border_width (GTK_CONTAINER (cd->props), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (cd->props)->vbox), 2);
 		
 	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
 	                                  "gnome-clock.png", TRUE, NULL);
