@@ -113,7 +113,7 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 	bonobo_ui_component_set_prop (frame->priv->ui_component,
 				      "/commands/RemoveAppletFromPanel",
 				      "sensitive",
-				      removable ? "1" : "0",
+				      locked ? "0" : (removable ? "1" : "0"),
 				      NULL);
 
 	bonobo_ui_component_set_prop (frame->priv->ui_component,
