@@ -577,8 +577,12 @@ panel_widget_cadd (GtkContainer *container,
 
 	p = g_object_get_data (G_OBJECT(widget),
 			       PANEL_APPLET_ASSOC_PANEL_KEY);
-	if(p)
-		run_up_forbidden(p,add_panel_to_forbidden);
+	if (p) {
+		panel_toplevel_attach_to_widget (p->toplevel,
+						 PANEL_WIDGET (container)->toplevel,
+						 widget);
+		run_up_forbidden (p, add_panel_to_forbidden);
+	}
 }
 
 static void
@@ -597,8 +601,10 @@ panel_widget_cremove (GtkContainer *container, GtkWidget *widget)
 	p = g_object_get_data (G_OBJECT (widget),
 				 PANEL_APPLET_ASSOC_PANEL_KEY);
 
-	if (p != NULL)
+	if (p != NULL) {
+		panel_toplevel_detach (p->toplevel);
 		run_up_forbidden (p, remove_panel_from_forbidden);
+	}
 
 	panel_widget_reset_focus (container, widget);
 
@@ -1676,6 +1682,11 @@ panel_widget_destroy (GtkObject *obj)
 
 	panels = g_slist_remove (panels, panel);
 
+	if (panel->master_widget != NULL)
+		g_object_set_data (G_OBJECT (panel->master_widget),
+				   PANEL_APPLET_ASSOC_PANEL_KEY,
+				   NULL);
+
 	if (GTK_OBJECT_CLASS (panel_widget_parent_class)->destroy)
 		GTK_OBJECT_CLASS (panel_widget_parent_class)->destroy (obj);
 }
@@ -1836,7 +1847,8 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 		gdk_flush ();
 
 		if (status != GDK_GRAB_SUCCESS) {
-			g_warning (G_STRLOC ": failed to grab pointer");
+			g_warning (G_STRLOC ": failed to grab pointer (errorcode: %d)",
+				   status);
 			panel_widget_applet_drag_end (panel);
 		}
 	}
@@ -2214,9 +2226,13 @@ panel_widget_applet_event(GtkWidget *widget, GdkEvent *event)
 
 			if ( ! commie_mode &&
 			    bevent->button == 2) {
+				guint32 time_ = bevent->time;
+				/* time on sent events seems to be bogus */
+				if (bevent->send_event)
+					time_ = GDK_CURRENT_TIME;
 				/* Start drag */
 				panel_widget_applet_drag_start (
-					panel, widget, PW_DRAG_OFF_CURSOR, bevent->time);
+					panel, widget, PW_DRAG_OFF_CURSOR, time_);
 				return TRUE;
 			}
 
