@@ -31,7 +31,11 @@ G_BEGIN_DECLS
 #define PANEL_APPLET_FORBIDDEN_PANELS "panel_applet_forbidden_panels"
 #define PANEL_APPLET_DATA "panel_applet_data"
 
+#ifndef TYPEDEF_PANEL_WIDGET
 typedef struct _PanelWidget		PanelWidget;
+#define TYPEDEF_PANEL_WIDGET
+#endif /* TYPEDEF_PANEL_WIDGET */
+
 typedef struct _PanelWidgetClass	PanelWidgetClass;
 
 typedef struct _AppletRecord		AppletRecord;
@@ -46,9 +50,6 @@ struct _AppletData
 	int		cells;
 	int             min_cells;
 
-	gboolean        expand_major;
-	gboolean        expand_minor;
-	gboolean        locked;
 	
 	gboolean	dirty;
 
@@ -63,6 +64,11 @@ struct _AppletData
 	int *           size_hints; 
 	int             size_hints_len;
   
+	guint           size_constrained : 1;
+	guint           expand_major : 1;
+	guint           expand_minor : 1;
+	gboolean        locked : 1;
+  
 };
 
 struct _PanelWidget
@@ -183,18 +189,27 @@ void            panel_widget_focus              (PanelWidget *panel);
 
 PanelOrientation panel_widget_get_applet_orientation (PanelWidget *panel);
 
-void         panel_widget_set_applet_expandable (PanelWidget *panel,
-						 GtkWidget   *applet,
-						 gboolean     major,
-						 gboolean     minor);
 
-void         panel_widget_set_applet_locked     (PanelWidget *panel,
-						 GtkWidget   *applet,
-						 gboolean     locked);
-gboolean     panel_widget_get_applet_locked     (PanelWidget *panel,
-						 GtkWidget   *applet);
-gboolean     panel_widget_toggle_applet_locked  (PanelWidget *panel,
-						 GtkWidget   *applet);
+void     panel_widget_set_applet_size_constrained (PanelWidget *panel,
+						   GtkWidget   *applet,
+						   gboolean     size_constrained);
+void     panel_widget_set_applet_expandable       (PanelWidget *panel,
+						   GtkWidget   *applet,
+						   gboolean     major,
+						   gboolean     minor);
+void     panel_widget_set_applet_size_hints       (PanelWidget *panel,
+						   GtkWidget   *applet,
+						   int         *size_hints,
+						   int          size_hints_len);
+
+void     panel_widget_set_applet_locked           (PanelWidget *panel,
+						   GtkWidget   *applet,
+						   gboolean     locked);
+gboolean panel_widget_get_applet_locked           (PanelWidget *panel,
+						   GtkWidget   *applet);
+gboolean panel_widget_toggle_applet_locked        (PanelWidget *panel,
+						   GtkWidget   *applet);
+
 
 G_END_DECLS
 
